"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = childAncestor;

var _isRootJsx = require("./is-root-jsx");

var _isRootJsx2 = _interopRequireDefault(_isRootJsx);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// It is only a child if it is a descendant of a JSX element but not a
// JSX Attribute.
function descendant(path) {
  var isChild = false;

  var direct = directChild(path);
  if (direct) {
    return direct;
  }

  path.findParent(function (path) {
    if (path.isJSXAttribute()) {
      // Stop traversing, we are not a child.
      return true;
    }

    if (path.isJSXElement()) {
      if (!directChild(path)) {
        // This element is the top of a tree of JSX elements
        // If it's the root tree, we are a child.
        isChild = (0, _isRootJsx2.default)(path);
        return true;
      }
    }
  });

  if (!isChild) {
    return null;
  }

  return path.findParent(function (path) {
    return !path.parentPath || path.parentPath.isJSX();
  });
}

// It is only a child if it's immediate parent is a JSX element,
// or it is an ExpressionContainer who's parent is.
function directChild(path) {
  var isChild = false;
  var child = path;

  path.findParent(function (path) {
    if (path.isJSXElement()) {
      isChild = true;
      return true;
    }

    if (path.isJSXExpressionContainer()) {
      // Defer to what the parent is.
      return false;
    }

    child = path;
    return true;
  });

  return isChild ? child : null;
}

// Detects if this element is not a child of another JSX element
function childAncestor(path, _ref) {
  var opts = _ref.opts;

  return (opts.fastRoot ? descendant : directChild)(path);
}